<?php

namespace Drupal\instagram_field\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Instagram Field Callback Controller.
 */
class CallbackController extends ControllerBase {

  /**
   * Callback from instagram auth with accesstoken.
   */
  public function callback() {
    $config = \Drupal::getContainer()->get('config.factory')
      ->getEditable('config.instagram_field');
    \Drupal::service('page_cache_kill_switch')->trigger();
    if (\Drupal::request()->query->get('code') === NULL) {
      $err_msg = t("instagramautherror: Error no code");
      \Drupal::logger('instagram_field')->error($err_msg);
      drupal_set_message($err_msg, 'error');
      return [
        '#type' => 'markup',
        '#markup' => '',
        '#cache' => [
          'max-age' => 0,
        ],
      ];
    }
    elseif (substr(\Drupal::request()->query->get('code'), 0, 5) == 'error') {
      $err_msg = t("instagramautherror: @code", [
        '@code' => \Drupal::request()->query->get('code'),
      ]);
      \Drupal::logger('instagram_field')->error($err_msg);
      drupal_set_message($err_msg, 'error');
      return [
        '#type' => 'markup',
        '#markup' => '',
        '#cache' => [
          'max-age' => 0,
        ],
      ];
    }
    else {
      $client = \Drupal::httpClient();
      $request = $client
        ->post('https://api.instagram.com/oauth/access_token', [
          'form_params' => [
            'client_id' => trim($config->get('clientid')),
            'client_secret' => trim($config->get('clientsecret')),
            'grant_type' => 'authorization_code',
            'redirect_uri' => \Drupal::request()->getSchemeAndHttpHost() .
            '/_instagram_field_callback',
            'code' => preg_replace('/[^A-Fa-f0-9]/', '', \Drupal::request()->query->get('code')),
          ],
        ]);
      $result = json_decode($request->getBody());
      $config->set('accesstoken', preg_replace('/[^A-Fa-f0-9.]/', '', $result->access_token));
      $config->save();
    }

    $response = new RedirectResponse('/admin/config/services/instagram_field');
    $response->send();
    return $response;
  }

}
