<?php

namespace Drupal\instagram_field\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\ConfigFormBase;

/**
 * Defines a form to configure module settings for instagram field.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'instagram_field.settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['config.instagram_field'];
  }

  /**
   * Settings form.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['#attached']['library'][] =
      'instagram_field/instagram_field-scripts';

    $config = $this->config('config.instagram_field');
    $form['callbackurl'] = [
      '#type' => 'item',
      '#title' => t('Callback URL (readonly)'),
      '#description' => \Drupal::request()->getSchemeAndHttpHost() .
      '/_instagram_field_callback',
    ];
    $form['clientid'] = [
      '#type' => 'textfield',
      '#title' => t('Instagram client ID'),
      '#translatable' => TRUE,
      '#default_value' => $config->get('clientid') ?
      $config->get('clientid') : '',
    ];
    $form['clientsecret'] = [
      '#type' => 'textfield',
      '#title' => t('Instagram client secret'),
      '#translatable' => TRUE,
      '#default_value' => $config->get('clientsecret') ?
      $config->get('clientsecret') : '',
    ];
    $form['auth'] = [
      '#type' => 'button',
      '#value' => t('Authenticate'),
      '#ajax' => [],
    ];

    $form['accesstoken'] = [
      '#type' => 'textfield',
      '#title' => t('Instagram access token'),
      '#translatable' => TRUE,
      '#default_value' => $config->get('accesstoken') ?
      $config->get('accesstoken') : '',
    ];

    $form['imageresolution'] = [
      '#type' => 'select',
      '#title' => t('Image resolution'),
      '#maxlength' => 255,
      '#default_value' => $config->get('imageresolution') ?
      $config->get('imageresolution') : 'standard_resolution',
      '#translatable' => TRUE,
      '#options' => [
        'thumbnail' => 'Thumbnail (150x150)',
        'low_resolution' => 'Low (320x320)',
        'standard_resolution' => 'Standard (640x640)',
      ],
    ];
    $form['cachetime'] = [
      '#type' => 'number',
      '#title' => t('Cache time in minutes'),
      '#translatable' => TRUE,
      '#default_value' => $config->get('cachetime') ?
      $config->get('cachetime') : '60',
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * Settings form submit.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = \Drupal::getContainer()->get('config.factory')
      ->getEditable('config.instagram_field');

    $config->set('clientid', $form_state->getValue('clientid'));
    $config->set('clientsecret', $form_state->getValue('clientsecret'));
    $config->set('accesstoken', $form_state->getValue('accesstoken'));

    $config->set('imageresolution', $form_state->getValue('imageresolution'));
    $config->set('cachetime', $form_state->getValue('cachetime'));

    $config->save();

    return parent::submitForm($form, $form_state);
  }

  /**
   * Ajax auth callback.
   */
  public function ajaxAuthCallback(array &$form, FormStateInterface &$form_state) {
    $uri = 'https://api.instagram.com/oauth/authorize/?client_id=' .
      $form_state->getValue('clientid') . '&redirect_uri=' .
      urlencode(\Drupal::request()->getSchemeAndHttpHost() .
      '/_instagram_field_callback') . '&response_type=code';
    $options = [
      'query' => [],
    ];

    $form_state->setRebuild(TRUE);
    return $form_state->getValue('clientid');
  }

}
